import json
import requests
from datetime import datetime, date
import time
from dateutil.relativedelta import relativedelta
import logging
import pandas as pd
import sys
import re
import pytz
import base64

regex_pattern = r'^[A-Z]{2}[0-9]{9}$'

today = date.today()
today_logs = today.strftime("%d_%m_%Y")

logging.basicConfig(filename=("logs/vida_to_apilo_" + today_logs +".log"), level=logging.INFO, format='%(asctime)s %(levelname)s:%(message)s', datefmt='%Y-%m-%d %H:%M:%S')

"""
pobieranie zamówień z vidy sprzed 2h
sprawdzanie czy zamówienie jest z apilo (zamówienie jest z apilo jesli w comments_customer (do potwierdzenia czy cos sie nie zmienilo) w zwrotce od vidy w tym polu pierwsze 2 znaki to duze litery)
wrzucenie zamówień do csvki
sprawdzenie zamówień z csvki jedno po drugim (jak aw) czy już jest tracking i wrzucenie go do csvki i na apilo
"""

def get_orders_from_vida(account, params):
    print('get_orders_from_vida')
    logging.info('get_orders_from_vida')

    url = "https://b2b.vidaxl.com/api_customer/orders.json"
    
    r = requests.get(url, auth=requests.auth.HTTPBasicAuth(username=account['email'], password=account['password']), params = params)
    
    logging.info(r.status_code)
    if 'application/json' in r.headers['Content-Type']:
        return r.json(), r.ok
    else:
        return json.loads(r.text), r.ok

def get_orders_from_csv(account):
    logging.info("get_orders_from_csv")
    print("get_orders_from_csv")
    orders = pd.read_csv(f"orders_{account['company']}.csv", delimiter = ';', keep_default_na=False).to_dict('records')
    return orders

def check_duplicates(orders, account):
    logging.info('check_duplicates')
    print("check_duplicates")
    saved_orders = get_orders_from_csv(account)
    saved_orders_ids = []
    orders_to_append_to_csv = []
    
    for x in saved_orders:
        saved_orders_ids.append(str(x['vida_order_id']))
    
    for z in orders:
        if re.match(regex_pattern, str(z['order']['comments_customer'])):
            if str(z['order']['id']) not in saved_orders_ids:
                orders_to_append_to_csv.append(z['order'])
    return orders_to_append_to_csv

def save_orders_to_csv(orders, mode, account):
    logging.info("save_orders_to_csv")
    print("save_orders_to_csv")
    df = []
    if mode == 'a':
        for order in orders:
            order_to_save = {
                'vida_order_id': order['id'],
                'apilo_order_id': order['comments_customer'],
                'tracking': '',
                'status': order['status_order_name'],
                'status_changes': order['status_order_name'],
                'date': today_logs
            }
            logging.info(json.dumps(order_to_save, indent = 2))
            df.append(order_to_save)
        df = pd.DataFrame.from_dict(df)
        df.to_csv(f"orders_{account['company']}.csv", sep=';', mode = mode, index=False, header=False)
    elif mode == 'w':
        for order in orders:
            df.append(
                {
                    'vida_order_id': order['vida_order_id'],
                    'apilo_order_id': order['apilo_order_id'],
                    'tracking': order['tracking'],
                    'status': order['status'],
                    'status_changes': order['status_changes'],
                    'date': order['date']
                }
            )
        df = pd.DataFrame.from_dict(df)
        if not df.empty:
            df.to_csv(f"orders_{account['company']}.csv", sep=';', mode = mode, index=False, header=['vida_order_id', 'apilo_order_id', 'tracking', 'status', 'status_changes', 'date'])
 
def add_note_apilo(order, cod):
    print("add_note_to_apilo")
    logging.info("add_note_to_apilo")
    url = "https://shumee.apilo.com/rest/api/orders/{}/note/"
    headers = {
        'Authorization': f'Bearer {TOKEN}',
        'Accept': 'application/json',
        'Content-Type': 'application/json' 
    }
    if cod:
        payload = {
            "type": 2,
            "comment": f"Przesyłka pobraniowa: {order['id']} Tracking: {order['shipping_tracking']}"
        }
    else:
        payload = {
            "type": 2,
            "comment": f"{order['id']} STATUS: {order['status_order_name']}"
        }

    response = requests.post(url.format(order['comments_customer']), headers=headers, json=payload)
    logging.info(f"apilo_id: {order['comments_customer']}")
    logging.info(response.status_code)
    logging.info(response.text)
    if response.status_code == 200 or response.status_code == 201:
        try:
            return response.json()  
        except ValueError as e:
            logging.info(f"JSON decoding error: {str(e)}")
            return f"JSON decoding error: {str(e)}"
    else:
        logging.info(f"Request failed with status code: {response.status_code}\n{response.text}")
        return f"Request failed with status code: {response.status_code}\n{response.text}"
        
def check_order(order, account):
    print('check_order')
    logging.info('get_order_from_vida')

    url = "https://b2b.vidaxl.com/api_customer/orders.json"
    print(order['vida_order_id'])
    params = {
        "id_eq": order['vida_order_id'].replace('B2B', '')
    }
    r = requests.get(url, auth=requests.auth.HTTPBasicAuth(username=account['email'], password=account['password']), params = params)
    print(r.text)
    logging.info(r.status_code)
    order_to_log = {
        'vida_order_id': r.json()[0]['order']['id'],
        'apilo_order_id': r.json()[0]['order']['comments_customer'],
        'tracking': r.json()[0]['order']['shipping_tracking'],
        'status': r.json()[0]['order']['status_order_name'],
    }
    logging.info(json.dumps(order_to_log, indent = 2))
    return r.json()[0]['order']

def upload_tracking_to_apilo(order):
    print('upload_tracking_to_apilo')
    logging.info("upload_tracking_to_apilo")
    
    url = 'https://shumee.apilo.com/rest/api/orders/{}/shipment/'
    
    headers = {
        'Authorization': f'Bearer {TOKEN}',
        'Accept': 'application/json',
        'Content-Type': 'application/json' 
    }
    trackings = order['shipping_tracking'].split(',')

    current_id_external = 1
    now = datetime.now(pytz.utc)

    # Convert the current time to a specific timezone if needed
    # For example, convert to UTC+1 (which is CET or WAT)
    target_timezone = pytz.timezone('Etc/GMT-1')
    now_in_target_timezone = now.astimezone(target_timezone)

    # Format the datetime to the specified string format
    formatted_datetime = now_in_target_timezone.strftime('%Y-%m-%dT%H:%M:%S%z')
    for single_tracking in trackings:
        payload = {
            "idExternal": str(current_id_external),
            "tracking": single_tracking.strip(),
            "carrierProviderId": 25,
            "postDate": formatted_datetime
        }

        response = requests.post(url.format(order['comments_customer']), headers=headers, json=payload)
        logging.info(f"tracking: {single_tracking.strip()}")
        
        logging.info(response.status_code)
        logging.info(response.text)
        
        current_id_external += 1

def check_date(date_str):
    date_format = "%d_%m_%Y"
    log_date = datetime.strptime(date_str, date_format)
    # Get the current date
    current_date = datetime.now()

    # Calculate the date one month ago
    one_month_ago = current_date - relativedelta(months=1)
    if log_date > one_month_ago:
        return True
    return False
       
def check_statuses(account):
    print('check statuses')
    logging.info("check_statuses")
    orders = get_orders_from_csv(account)

    for existing_order in orders:
        # check orders max one month back
        if check_date(existing_order['date']):
            order = check_order(existing_order, account)
            if order['status_order_name'] != existing_order['status']:
                existing_order['status'] = order['status_order_name']
                existing_order['status_changes'] = existing_order['status_changes'] + f'->{order["status_order_name"]}'
                add_note_apilo(order, cod = False)
            
            # check tracking
            if order['shipping_tracking']:
                if order['shipping_tracking'] != existing_order['tracking']:
                    existing_order['tracking'] = order['shipping_tracking']
                    # cod
                    if "1 maja 31/33" in order['order_products'][0]['order_product']['addressbook']['address'].lower():
                        add_note_apilo(order, cod = True)
                    else:
                        upload_tracking_to_apilo(order)
    return orders

def main():
    accounts = [
        {
            'email': 'michal@shumee.pl',
            'password': '149dd9e9-665b-45bb-9a60-14bc4c598720',
            'company': 'shumee'
        },
        {
            'email': 'gstore@greatstore.pl',
            'password': '4d262cbf-33ff-4f8f-b5aa-219a3844d4ea',
            'company': 'greatstore'
        },
        {
            'email': 'greece@shumee.pl',
            'password': '4de3baa4-32ba-48d7-9865-006b6aa8dc0c',
            'company': 'shumee_greece'
        },
        {
            'email': 'estonia@shumee.pl',
            'password': '9aafd959-8303-400c-8ef5-795726524367',
            'company': 'greatstore_estonia'
        },
        {
            'email': 'finland@greatstore.pl',
            'password': 'bf841c8a-41a7-44f0-859f-990ffba0aeb0',
            'company': 'greatstore_finland'
        },
        {
            'email': 'lithuania@shumee.pl',
            'password': '9967f9af-287c-476f-aac6-1fc5cdc3b2ce',
            'company': 'greatstore_lithuania'
        },
        {
            'email': 'latvia@shumee.pl',
            'password': 'c256bed2-cbe4-4a5c-9a7d-d28c12d12446',
            'company': 'greatstore_latvia'
        }
    ]
    
    params = {
        "submitted_at_gteq": today.strftime("%Y%m%d")
    }
    if len(sys.argv) > 1:
        args_list = sys.argv[1].split(",")
        for acc in accounts:
            if acc['company'] == args_list[0]:
                account = acc
        order_ids = args_list[1:]
        for order_id in order_ids:
            params = {
                "id_eq": order_id
            }
            temp = get_orders_from_vida(account, params)
            if temp[1]:
                orders.append(temp[0])
    else:
        for account in accounts:
            orders = []
            
            temp = get_orders_from_vida(account, params)
            if temp[1]:
                orders += temp[0]
                
            orders = check_duplicates(orders, account)  
            for order in orders:
                add_note_apilo(order, cod = False)
            save_orders_to_csv(orders, 'a', account)
            
            orders = check_statuses(account)

            save_orders_to_csv(orders, 'w', account)

if __name__ == "__main__":
    start_time = time.time()
    with open('/usr/home/dkkapusta1997/domains/dkkapusta1997.usermd.net/public_html/others/apilo_token_refresher/response.json', 'r') as file:
        response_data = json.load(file)
        TOKEN = response_data.get('accessToken', '')
    main()
    end_time = time.time()
    with open("/usr/home/dkkapusta1997/domains/dkkapusta1997.usermd.net/public_html/warehouses.csv", 'a') as f:
        f.write(datetime.now().strftime("%d-%m-%y_%H:%M")+";vida_to_apilo;"+str(round(end_time-start_time, 2))+"\n")








